﻿/* Author: Sebastian Dusch (http://die-rooter.de/ITworks)
* Date: 26-Nov-2009
* The code you have here is licensed as "Creative Commons NC-BY-SA"
* Please contact me at neiky(at)premiumpixel.com if you want to use this code
* in commercial projects.
* This file is licensed under Creative Commons Namensnennung-Keine kommerzielle Nutzung-Weitergabe unter gleichen Bedingungen 3.0 Deutschland license.
*/ 
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace System.Windows.Controls.Extensions
{
    public class SharedSize : DependencyObject
    {
        /// <summary>
        /// Sets or gets the name of the shared size group.
        /// </summary>
        public static readonly DependencyProperty IsSharedSizeScopeProperty = DependencyProperty.RegisterAttached(
                    "IsSharedSizeScope",
                    typeof(bool),
                    typeof(SharedSize),
                    new PropertyMetadata(false));

        /// <summary>
        /// Sets the IsSharedSizeScope for the provided DependencyObject
        /// </summary>
        /// <param name="target">The target</param>
        /// <param name="isSharedSizeScope">the value</param>
        public static void SetIsSharedSizeScope(DependencyObject target, bool isSharedSizeScope)
        {
            FrameworkElement element = target as FrameworkElement;
            if (element != null)
            {
                target.SetValue(SharedSize.IsSharedSizeScopeProperty, isSharedSizeScope);
                SharedSizeManager.Instance.AddScope(element);
            }
            else
            {
                throw new NotSupportedException("The IsSharedSizeScope property is only supported on FrameworkElement class");
            }
        }

        /// <summary>
        /// Returns the IsSharedSizeScope for the provided DependencyObject
        /// </summary>
        /// <param name="target">The object that has the value</param>
        /// <returns>The IsSharedSizeScope value of the object</returns>
        public static bool GetIsSharedSizeScope(DependencyObject target)
        {
            return (bool)target.GetValue(SharedSize.IsSharedSizeScopeProperty);
        }

        /// <summary>
        /// Sets or gets the name of the shared size group.
        /// </summary>
        public static readonly DependencyProperty SharedSizeGroupProperty = DependencyProperty.RegisterAttached(
                    "SharedSizeGroup",
                    typeof(string),
                    typeof(SharedSize),
                    new PropertyMetadata(""));

        /// <summary>
        /// Sets the shared size group property for the provided DependencyObject. 
        /// The DependencyObject must be of type RowDefinition!
        /// </summary>
        /// <param name="obj">The object</param>
        /// <param name="groupName">The group name</param>
        /// <exception cref="System.NotSupportedException">If the provided DependencyObject 
        /// is not of type RowDefinition</exception>
        public static void SetSharedSizeGroup(DependencyObject target, string groupName)
        {
            RowDefinition row = target as RowDefinition;
            ColumnDefinition column = target as ColumnDefinition;

            if (row != null)
            {
                if (row.Height.IsStar)
                {
                    row.Height = new GridLength(0, GridUnitType.Auto);
                }
                target.SetValue(SharedSize.SharedSizeGroupProperty, groupName);
                SharedSizeManager.Instance.AddRow(row);
            }
            else if (column != null)
            {
                if (column.Width.IsStar)
                {
                    column.Width = new GridLength(0, GridUnitType.Auto);
                }
                target.SetValue(SharedSize.SharedSizeGroupProperty, groupName);
                SharedSizeManager.Instance.AddColumn(column);
            }
            else
            {
                throw new NotSupportedException("The SharedSizeGroup property is only supported on RowDefinition or ColumnDefinition class");
            }

        }

        /// <summary>
        /// Gets the shared size group property for the provided DependencyObject.
        /// </summary>
        /// <param name="obj">The object</param>
        /// <returns>The group name</returns>
        public static string GetSharedSizeGroup(DependencyObject target)
        {
            return (string)target.GetValue(SharedSize.SharedSizeGroupProperty);
        }
    }
}
