﻿/* Author: Sebastian Dusch (http://die-rooter.de/ITworks)
* Date: 26-Nov-2009
* The code you have here is licensed as "Creative Commons NC-BY-SA"
* Please contact me at neiky(at)premiumpixel.com if you want to use this code
* in commercial projects.
* This file is licensed under Creative Commons Namensnennung-Keine kommerzielle Nutzung-Weitergabe unter gleichen Bedingungen 3.0 Deutschland license.
*/ 
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Controls.Extensions.EventArgs;
using System.Linq;

namespace System.Windows.Controls.Extensions
{
    public class SharedSizeGrid
    {
        public event EventHandler<ColumnChangedEventArgs> ColumChanged;
        public event EventHandler<RowChangedEventArgs> RowChanged;

        public List<SharedSizeRow> Rows { get; private set; }
        public List<SharedSizeColumn> Columns { get; private set; }
        public Grid TargetObject { get; private set; }

        public SharedSizeGrid(Grid targetObject)
        {
            this.Rows = new List<SharedSizeRow>();
            this.Columns = new List<SharedSizeColumn>();

            this.TargetObject = targetObject;
            this.TargetObject.SizeChanged += new SizeChangedEventHandler(Grid_SizeChanged);
            this.TargetObject.LayoutUpdated += new EventHandler(Grid_LayoutUpdated);

            /* To prevent raise conditions, we add one event handler for our own events */
            this.ColumChanged += new EventHandler<ColumnChangedEventArgs>(Dummy_ColumChanged);
            this.RowChanged += new EventHandler<RowChangedEventArgs>(Dummy_RowChanged);
        }

        /// <summary>
        /// Checks, if some of the rows or columns have changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Grid_LayoutUpdated(object sender, System.EventArgs e)
        {
            Debug.WriteLine("Grid_LayoutUpdated");
            CheckChanges();
        }

        /// <summary>
        /// Checks, if some of the rows or columns have changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Grid_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            Debug.WriteLine("Grid_SizeChanged");
            CheckChanges();
        }

        /// <summary>
        /// Checks, if some of the rows or columns have changed and
        /// raises the according events
        /// </summary>
        private void CheckChanges()
        {
            bool rowsChanged = false;
            bool columnsChanged = false;

            foreach (SharedSizeRow row in this.Rows)
            {
                if (row.HasChanged)
                {
                    rowsChanged = true;
                    this.OnRowChanged(row);
                }
            }

            foreach (SharedSizeColumn column in this.Columns)
            {
                if (column.HasChanged)
                {
                    columnsChanged = true;
                    this.OnColumnChanged(column);
                }
            }

            Debug.WriteLine("\t\tRows: " + rowsChanged + " Columns: " + columnsChanged);
        }

        /* Dummy methods to prevent raise conditions */
        private void Dummy_ColumChanged(object sender, ColumnChangedEventArgs e) { }
        private void Dummy_RowChanged(object sender, RowChangedEventArgs e) { }

        /// <summary>
        /// Raises the ColumnChanged event
        /// </summary>
        /// <param name="column">The column that changed</param>
        protected virtual void OnColumnChanged(SharedSizeColumn column)
        {
            /* We dont have to check for null here because there is always the own delegate registered */
            this.ColumChanged(this, new ColumnChangedEventArgs(column));
        }

        /// <summary>
        /// Raises the RowChanged event
        /// </summary>
        /// <param name="column">The column that changed</param>
        protected virtual void OnRowChanged(SharedSizeRow row)
        {
            /* We dont have to check for null here because there is always the own delegate registered */
            this.RowChanged(this, new RowChangedEventArgs(row));
        }

        /// <summary>
        /// Gets the rows with the provided group name
        /// </summary>
        /// <param name="groupName"></param>
        /// <returns></returns>
        internal List<SharedSizeRow> GetRowsForGroup(string groupName)
        {
            return this.Rows.Where(row => row.GroupName.Equals(groupName)).ToList();
        }

        /// <summary>
        /// Gets the columns with the provided group name
        /// </summary>
        /// <param name="groupName"></param>
        /// <returns></returns>
        internal List<SharedSizeColumn> GetColumnsForGroup(string groupName)
        {
            return this.Columns.Where(column => column.GroupName.Equals(groupName)).ToList();
        }
    }
}
