﻿/* Author: Sebastian Dusch (http://die-rooter.de/ITworks)
* Date: 26-Nov-2009
* The code you have here is licensed as "Creative Commons NC-BY-SA"
* Please contact me at neiky(at)premiumpixel.com if you want to use this code
* in commercial projects.
* This file is licensed under Creative Commons Namensnennung-Keine kommerzielle Nutzung-Weitergabe unter gleichen Bedingungen 3.0 Deutschland license.
*/ 
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Windows.Controls.Extensions.EventArgs;
using System.Diagnostics;
using System.Linq;

namespace System.Windows.Controls.Extensions
{
    public class SharedSizeScope
    {
        public FrameworkElement HostElement { get; private set; }
        public ObservableCollection<SharedSizeGrid> Children { get; private set; }

        public SharedSizeScope(FrameworkElement hostElement)
        {
            this.Children = new ObservableCollection<SharedSizeGrid>();
            this.Children.CollectionChanged += this.Children_CollectionChanged;
            this.HostElement = hostElement;
        }

        /// <summary>
        /// Registers the delgates for added rows and columns for the new items and
        /// removes the delegates for old items.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Children_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
            {
                foreach (object oGrid in e.NewItems)
                {
                    SharedSizeGrid grid = oGrid as SharedSizeGrid;
                    if (grid != null)
                    {
                        grid.RowChanged += this.ChildGrid_RowChanged;
                        grid.ColumChanged += this.ChildGrid_ColumnChanged;
                    }
                    else
                    {
                        throw new ArgumentException("Only SharedSizeGrid objects can be added to the collection." +
                            " Actually, this error should never happen!", "e");
                    }
                }
            }

            if (e.OldItems != null)
            {
                foreach (object oGrid in e.OldItems)
                {
                    SharedSizeGrid grid = oGrid as SharedSizeGrid;
                    if (grid != null)
                    {
                        grid.RowChanged -= this.ChildGrid_RowChanged;
                        grid.ColumChanged -= this.ChildGrid_ColumnChanged;
                    }
                    else
                    {
                        throw new ArgumentException("Only SharedSizeGrid objects can be removed from the collection." +
                            " Actually, this error should never happen!", "e");
                    }
                }
            }
        }

        /// <summary>
        /// Layouts the rows of the same group
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ChildGrid_RowChanged(object sender, RowChangedEventArgs e)
        {
            Debug.WriteLine("Row changed from " + e.Row.CurrentValue + " to " + e.Row.TargetObject.ActualHeight);

            /* We cannot start layouting until all rows are layouted or have an fixes height */

            bool allRowsLayouted = true;
            List<SharedSizeRow> groupRows = new List<SharedSizeRow>();
            foreach (SharedSizeGrid grid in this.Children)
            {
                List<SharedSizeRow> gridRows = grid.GetRowsForGroup(e.Row.GroupName);
                SharedSizeRow autoRow = gridRows.FirstOrDefault(
                        row => row.TargetObject.Height.IsAuto && row.TargetObject.ActualHeight == 0);

                if (autoRow != null)
                {
                    /* There is still a row that was not layouted */
                    //FIXME: not really true because the row's height can actually be 0 even after layouting
                    allRowsLayouted = false;
                }
                else
                {
                    groupRows.AddRange(gridRows);
                }
            }

            if (allRowsLayouted)
            {
                double maxHeight = groupRows.Max(row => row.TargetObject.ActualHeight);

                foreach (SharedSizeRow row in groupRows)
                {
                    row.SetValue(maxHeight);
                }
            }
        }

        /// <summary>
        /// Layouts the columns of the same group
        /// </summary>
        void ChildGrid_ColumnChanged(object sender, ColumnChangedEventArgs e)
        {
            Debug.WriteLine("Column changed from " + e.Column.CurrentValue + " to " + e.Column.TargetObject.ActualWidth);

            bool allColumnsLayouted = true;
            List<SharedSizeColumn> groupColumns = new List<SharedSizeColumn>();
            foreach (SharedSizeGrid grid in this.Children)
            {
                List<SharedSizeColumn> gridColumns = grid.GetColumnsForGroup(e.Column.GroupName);
                SharedSizeColumn autoColumn = gridColumns.FirstOrDefault(
                        row => row.TargetObject.Width.IsAuto && row.TargetObject.ActualWidth == 0);

                if (autoColumn != null)
                {
                    /* There is still a row that was not layouted */
                    //FIXME: not really true because the column's width can actually be 0 even after layouting
                    allColumnsLayouted = false;
                }
                else
                {
                    groupColumns.AddRange(gridColumns);
                }
            }

            if (allColumnsLayouted)
            {
                double maxWidth = groupColumns.Max(column => column.TargetObject.ActualWidth);

                foreach (SharedSizeColumn column in groupColumns)
                {
                    column.SetValue(maxWidth);
                }
            }
        }
    }
}
