﻿/* Author: Sebastian Dusch (http://die-rooter.de/ITworks)
* Date: 26-Nov-2009
* The code you have here is licensed as "Creative Commons NC-BY-SA"
* Please contact me at neiky(at)premiumpixel.com if you want to use this code
* in commercial projects.
* This file is licensed under Creative Commons Namensnennung-Keine kommerzielle Nutzung-Weitergabe unter gleichen Bedingungen 3.0 Deutschland license.
*/
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace System.Windows.Controls.Extensions
{
    public static class UIHelper
    {        
        internal static List<SharedSizeGrid> FindGridsInScope(this DependencyObject element)
        {
            List<SharedSizeGrid> children = new List<SharedSizeGrid>();

            Grid currentGrid = element as Grid;
            if (currentGrid != null)
            {
                SharedSizeGrid sharedSizeGrid = new SharedSizeGrid(currentGrid);
                sharedSizeGrid.Rows.AddRange(currentGrid.GetRowsWithSharedSize());
                sharedSizeGrid.Columns.AddRange(currentGrid.GetColumnsWithSharedSize());

                /* Check if this Grid has some magic rows or columns */
                if (sharedSizeGrid.Rows.Count > 0 || sharedSizeGrid.Columns.Count > 0)
                {
                    children.Add(sharedSizeGrid);
                }
            }

            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(element); i++)
            {
                DependencyObject child = VisualTreeHelper.GetChild(element, i);

                if (!SharedSize.GetIsSharedSizeScope(child))
                {
                    children.AddRange(child.FindGridsInScope());
                }
            }


            return children;
        }

        internal static List<SharedSizeRow> GetRowsWithSharedSize(this Grid element)
        {
            List<SharedSizeRow> sharedRows = new List<SharedSizeRow>();

            foreach (RowDefinition row in (element as Grid).RowDefinitions)
            {
                if (!String.IsNullOrEmpty(SharedSize.GetSharedSizeGroup(row)))
                {
                    sharedRows.Add(new SharedSizeRow(row));
                }
            }

            return sharedRows;
        }

        internal static bool HasRowsWithSharedSize(this Grid element)
        {
            bool hasRows = false;

            foreach (RowDefinition row in (element as Grid).RowDefinitions)
            {
                if (!String.IsNullOrEmpty(SharedSize.GetSharedSizeGroup(row)))
                {
                    hasRows = true;
                    break;
                }
            }

            return hasRows;
        }

        internal static List<SharedSizeColumn> GetColumnsWithSharedSize(this Grid element)
        {
            List<SharedSizeColumn> sharedColumns = new List<SharedSizeColumn>();

            foreach (ColumnDefinition column in (element as Grid).ColumnDefinitions)
            {
                if (!String.IsNullOrEmpty(SharedSize.GetSharedSizeGroup(column)))
                {
                    sharedColumns.Add(new SharedSizeColumn(column));
                }
            }

            return sharedColumns;
        }

        internal static bool HasColumnsWithSharedSize(this Grid element)
        {
            bool hasColumns = false;

            foreach (ColumnDefinition column in (element as Grid).ColumnDefinitions)
            {
                if (!String.IsNullOrEmpty(SharedSize.GetSharedSizeGroup(column)))
                {
                    hasColumns = true;
                    break;
                }
            }

            return hasColumns;
        }
    }
}
